import attr
from bowler import SYMBOL
from bowler import TOKEN
from bowler.helpers import find_first
from fissix.fixer_util import Name
from fissix.pytree import Leaf
from fissix.pytree import Node

from . import MARKER as _MARKER

MARKER = "pytest.mark." + _MARKER


@attr.s(frozen=True)
class TestFunc:
    filename = attr.ib(type=str)
    selector = attr.ib(type=str)

    @classmethod
    def from_pytest_results_line(cls, line):
        assert line.startswith(("E", "F"))

        line = line.split(" ", 1)[1]
        filename, selector = line.split("::", 1)

        # Remove parametrization from the test name
        selector = selector.split("[", 1)[0]

        return cls(filename=filename, selector=selector)

    @classmethod
    def from_node(cls, node, filename):
        name_parts = []

        while node:
            possible_name = node.children[1]
            if isinstance(possible_name, Leaf) and possible_name.type == TOKEN.NAME:
                name_parts.append(possible_name.value)

            node = node.parent

        selector = "::".join(reversed(name_parts))
        return cls(filename=filename, selector=selector)


def _get_indent(node):
    """Determine the indentation level of ``node``."""
    indent = None
    while node:
        indent = find_first(node, TOKEN.INDENT)
        if indent is not None:
            indent = indent.value
            break

        node = node.parent

    return indent


def _get_marker(node):
    """
    If ``node`` is already decorated with ``MARKER``, return the
    ``SMYBOL.decorator`` ``Node`` for ``MARKER``.

    Else, return ``None``.
    """
    if node.parent.type == SYMBOL.decorated:
        child = node.parent.children[0]
        if child.type == SYMBOL.decorator:
            decorators = [child]
        elif child.type == SYMBOL.decorators:
            decorators = child.children
        else:
            raise NotImplementedError
        for decorator in decorators:
            name = decorator.children[1]
            assert name.type in {TOKEN.NAME, SYMBOL.dotted_name}

            if str(name) == MARKER:
                return decorator
    return None


def add_marker(node, capture, filename):
    """Add ``MARKER`` to the functions."""
    indent = _get_indent(node)

    decorated = Node(
        SYMBOL.decorated,
        [
            Node(
                SYMBOL.decorator,
                [Leaf(TOKEN.AT, "@"), Name(MARKER), Leaf(TOKEN.NEWLINE, "\n")],
            )
        ],
        prefix=node.prefix,
    )
    node.replace(decorated)
    decorated.append_child(node)

    if indent is not None:
        node.prefix = indent
    else:
        node.prefix = ""


def remove_marker(node, capture, filename):
    """Remove ``MARKER`` from the functions."""
    assert node.parent.type == SYMBOL.decorated

    decorated_node = node.parent

    marker_node = _get_marker(node)

    # Remove the marker from the AST
    marker_node.remove()

    has_other_decorators = any(
        child.type in {SYMBOL.decorator, SYMBOL.decorators}
        for child in decorated_node.children
    )

    if not has_other_decorators:
        # Set the function node's leading whitespace to the same as the marker
        # that was removed
        node.prefix = marker_node.prefix
        # Since there are no other decorators, replace the decorated node with
        # the function node
        decorated_node.replace(node)


def filter_not_already_marked(node, capture, filename):
    """Don't modify tests that are already marked with MARKER."""
    return not _get_marker(node)


def filter_already_marked(node, capture, filename):
    """Only modify tests that are already marked with MARKER."""
    return bool(_get_marker(node))


def filter_failing_tests(results_file):
    """
    Limit to test functions that failed or errored.

    :param file results_file: pytest --result-log file
    """
    testfuncs = {
        TestFunc.from_pytest_results_line(line.strip())
        for line in results_file
        if line.startswith(("F", "E"))
    }

    def inner(node, capture, filename):
        return TestFunc.from_node(node, filename) in testfuncs

    return inner



from bowler import Query
import click

from ._bowler import filter_already_marked
from ._bowler import filter_failing_tests
from ._bowler import MARKER
from ._bowler import remove_marker


@click.command(help="Remove %s from failing tests" % MARKER)
@click.option(
    "--result-log",
    type=click.File("r"),
    help="Path to pytest --result-log file",
    required=False,
)
@click.option("--interactive/--no-interactive", is_flag=True, default=True)
@click.argument("paths", type=click.Path(exists=True, dir_okay=True), nargs=-1)
def main(result_log, interactive, paths):
    query = Query(paths).select("funcdef").filter(filter_already_marked)
    if result_log:
        query = query.filter(filter_failing_tests(result_log))

    query.modify(remove_marker).write(interactive=interactive)


if __name__ == "__main__":
    main()



import sys

import pytest

from . import MARKER

PY3 = sys.version_info.major == 3


def pytest_configure(config):
    config.addinivalue_line(
        "markers", "%s: tests that are expected to fail on Python 3" % MARKER
    )


def pytest_collection_modifyitems(items):
    for item in items:
        if item.get_closest_marker(MARKER):
            item.add_marker(
                pytest.mark.xfail(
                    PY3, reason="test is not compatible with Python 3", strict=True
                )
            )



import codecs
import os
import re

from setuptools import find_packages
from setuptools import setup

NAME = "pytest-mark-no-py3"
PACKAGES = find_packages(where="src")
META_PATH = os.path.join("src", "pytest_mark_no_py3", "__init__.py")
CLASSIFIERS = [
    "Intended Audience :: Developers",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.7",
    "License :: OSI Approved :: MIT License",
]

INSTALL_REQUIRES = ["pytest"]
EXTRAS_REQUIRE = {"bowler": ["bowler", "click", "attrs"]}

ENTRY_POINTS = {"pytest11": ["mark_no_py3 = pytest_mark_no_py3.plugin"]}

HERE = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """
    Build an absolute path from *parts* and and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()


META_FILE = read(META_PATH)


def find_meta(meta):
    """Extract __*meta*__ from META_FILE."""
    meta_match = re.search(
        r"^__{meta}__ = ['\"]([^'\"]*)['\"]".format(meta=meta), META_FILE, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{meta}__ string.".format(meta=meta))


VERSION = find_meta("version")


setup(
    name=NAME,
    version=VERSION,
    classifiers=CLASSIFIERS,
    packages=PACKAGES,
    package_dir={"": "src"},
    include_package_data=True,
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    url="https://github.com/GoodRx/pytest-mark-no-py3",
    description="pytest plugin and bowler codemod to help migrate tests to Python 3",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    license="MIT",
    entry_points=ENTRY_POINTS,
)
